@import url("https://fonts.googleapis.com/css?family=Roboto:100,300,400,400i,500,500i,700,900");
@import url("https://cdn.materialdesignicons.com/2.0.46/css/materialdesignicons.min.css");

body {
    margin: 0;
    height: 100vh;
    font-family: Roboto;
}

.unstyled {
    list-style: none;
    padding: 0;
    margin: 0;
}

.terminal-mdc-navigation {
    width: 16.6em;
    background: black;
    border-right: 1px solid #979393;
    height: 100%;
    max-height: 100%;
    overflow: auto;
}

.terminal-worker-mdc-container {
    display: -ms-flexbox;
    display: flex;
    margin: auto;
    -ms-flex: 0 0 auto;
    flex: 0 0 auto;
    -ms-flex-wrap: wrap;
    flex-wrap: wrap;
    -ms-flex-pack: justify;
    justify-content: center;
    width: 100%;
    flex-direction: column;
}

.terminal-worker-mdc {
    border: 1px solid #ffffff20;
    background: #373738;
    color: #6c757d;
    display: -ms-inline-flexbox;
    display: inline-flex;
    -ms-flex-pack: distribute;
    justify-content: flex-start;
    min-height: 100px;
    min-width: 200px;
    margin: 8px;
    -ms-flex-align: center;
    align-items: center !important;
    width: 15em;
    position: relative;
    cursor: pointer;
}

.terminal-worker-mdc:hover {
    border-left: 3px solid #979393;
    margin-left: 6px;
}

.selected {
    outline: 1px ridge white;
    background: #373738;
}

.terminal-worker-mdc-icon {
    width: 50px;
    height: 50px;
    min-width: 50px;
    min-height: 50px;
    overflow: hidden;
    display: flex;
    align-items: center;
    justify-content: center;
    margin: 7px;
}

.terminal-mdc-card {
    text-overflow: ellipsis;
    overflow: hidden;
}

.terminal-mdc-card h4, .terminal-mdc-card p {
    margin: 0;
    padding: 1px;
    text-overflow: ellipsis;
    overflow: hidden;
    white-space: nowrap;
}

.terminal-mdc-card h4 {
    color: #fff;
    font-weight: normal;
}

.logo {
    text-transform: lowercase;
    font: 300 2em "Source Sans Pro", Helvetica, Arial, sans-serif;
    text-align: center;
    position: absolute;
    top: 50%;
    left: 50%;
    transform: translate(-50%, -50%);

}

.logo div {
    display: block;
    padding: 1em 0;
    text-decoration: none;
    transition: 0.15s linear color;
}

.logo span {
    font-weight: 700;
}

.main-container {
    width: 100%;
    height: 100vh;
    padding: 0;
    margin: 0 auto;
    display: flex;
    position: relative;
}

.disconnected-overlay {
    position: absolute;
    left: 0;
    right: 0;
    top: 0;
    bottom: 0;
    background: #6c7178e8;
    z-index: 9999;
    display: none;
    text-align: center;
}

.disconnected-overlay-popup {
    width: 230px;
    height: 20px;
    position: fixed;
    top: calc(50% - 25px);
    left: calc(50% - 50px);
    padding: 40px;
    border-radius: 2px;
    background: #4d5591d9;
}

.terminal-container {
    flex-grow: 1;
    color: #6e6e6e !important;
}

.terminal {
    overflow: hidden;
    position: relative;
    width: 100%;
    height: 100vh;
    background: #2a2830;
}

.close-terminal {
    position: absolute;
    top: -11px;
    right: -12px;
    display: block;
    width: 50px;
    height: 50px;
    font-size: 0;
}

.close-terminal:before,
.close-terminal:after {
    position: absolute;
    top: 50%;
    left: 50%;
    width: 5px;
    height: 20px;
    background-color: #F0F0F0;
    transform: rotate(45deg) translate(-50%, -50%);
    transform-origin: top left;
    content: '';
}

.close-terminal:after {
    transform: rotate(-45deg) translate(-50%, -50%);
}

.close-terminal:before,
.close-terminal:after {
    width: 2px;
}

.close-terminal:before,
.close-terminal:after {
    height: 12px;
}